<?php

/**
 * Расчитываем хэш тела запроса
 * @param string $body Тело запроса в строковом представлении (json)
 *
 * @return string
 */
function createBodyChecksum(string $body): string
{
    return md5($body);
}

/**
 * Расчитываем подпись запроса
 *
 * @param string $secret Секретный ключ вашего канала
 * @param string $checkSum Рассчитанный хэш тела запроса
 * @param string $apiMethod Адрес вызываемого метода API
 * @param string $httpMethod HTTP метод запроса
 * @param string $contentType Передаваемый тип данных
 *
 * @return string
 */
function createSignature(
    string $secret,
    string $checkSum,
    string $apiMethod,
    string $httpMethod = 'POST',
    string $contentType = 'application/json'
): string {
    $str = implode("\n", [
        strtoupper($httpMethod),
        $checkSum,
        $contentType,
        date(DateTimeInterface::RFC2822),
        $apiMethod,
    ]);

    return hash_hmac('sha1', $str, $secret);
}

/**
 * Подготавливаем заголовки для запроса
 *
 * @param string $checkSum Рассчитанный хэш тела запроса
 * @param string $signature Рассчитанная подпись запроса
 * @param string $contentType Передаваемый тип данных
 *
 * @return array
 */
function prepareHeaderForCurl(
    string $checkSum,
    string $signature,
    string $contentType = 'application/json'
): array {
    $headers = [
        'Date' => date(DateTimeInterface::RFC2822),
        'Content-Type' => $contentType,
        'Content-MD5' => strtolower($checkSum),
        'X-Signature' => strtolower($signature),
        'User-Agent' => 'amoCRM-Chats-Doc-Example/1.0'
    ];

    foreach ($headers as $name => $value) {
        $curlHeaders[] = $name . ": " . $value;
    }

    return $curlHeaders;
}

/**
 * Выполняем запрос к API Чатов
 *
 * @param string $apiMethod Запрашиваемый метод API
 * @param string $requestBody Тело запроса
 * @param array $requestHeaders Заголовки запроса
 * @param string $httpMethod HTTP метод запроса
 */
function execCurl(
    string $apiMethod,
    string $requestBody,
    array $requestHeaders,
    string $httpMethod = 'POST'
): void {
    $curl = curl_init();
    $curlOptions = [
        CURLOPT_URL => 'https://amojo.amocrm.ru' . $apiMethod,
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_TIMEOUT => 5,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST => $httpMethod,
        CURLOPT_HTTPHEADER => $requestHeaders,
    ];

    if (!empty($requestBody)) {
        $curlOptions[CURLOPT_POSTFIELDS] = $requestBody;
    }

    curl_setopt_array($curl, $curlOptions);

    $response = curl_exec($curl);
    $error = curl_error($curl);
    $info = curl_getinfo($curl);
    curl_close($curl);
    if ($error) {
        echo "cURL Error #:" . $error;
    } else {
        echo "Status: " . $info['http_code'] . PHP_EOL;
        echo $response . PHP_EOL;
    }
}


